@@ -5,7 +5,7 @@ NULL
 
 #' Boxplot Function for Data Frame
 #'
-#' Boxplot for data.frame. Uses standard evaluation.
+#' Boxplot for data.frame. Creates a boxplot using \code{boxplot_panel} by default.
 #' @param x data.frame
 #' @param yvar y variable
 #' @param xvar x variable
@@ -13,11 +13,17 @@ NULL
 #' @param log whether to log transform numeric variable (auto-selected if NULL)
 #' @param horizontal whether box/whisker axis should be horizontal (numeric x, categorical y); defaults TRUE if (var[[2]] is numeric
 #' @param crit if log is NULL, log-transform if mean/median ratio for non-missing x is greater than this value
-#' @param ref optional reference line on numeric axis
+#' @param panel panel function
+#' @param ref optional reference line(s) on numeric axis; can be function(x = x, var = con, ...)
+#' @param ref.col color for reference line(s)
+#' @param ref.lty line type for reference line(s)
+#' @param ref.alpha transparency for reference line(s)
 #' @param nobs whether to include the number of observations under the category label
 #' @param na.rm whether to remove data points with one or more missing coordinates
-#' @param ylab passed to \code{\link[lattice]{bwplot}}
-#' @param xlab passed to \code{\link[lattice]{bwplot}}
+#' @param ylab y axis label
+#' @param xlab x axis label
+#' @param numlab numeric axis label; can be function(x = x, var = numvar, log = ylog, ...)
+#' @param catlab categorical axis label; can be function(x = x, var = catvar, ...)
 #' @param aspect passed to \code{\link[lattice]{bwplot}}
 #' @param main character, or a function of x, yvar, xvar, facets, and log
 #' @param sub character, or a function of x, yvar, xvar, facets, and log
@@ -25,11 +31,11 @@ NULL
 #' @export
 #' @importFrom rlang quos UQ
 #' @importFrom graphics boxplot
-#' @importFrom dplyr filter
 #' @import lattice
 #' @importFrom stats as.formula median cor loess.smooth density
 #' @family mixedvariate plots
 #' @family boxplot
+#' @family metaplot
 #' @examples
 #' library(magrittr)
 #' library(dplyr)
@@ -42,15 +48,21 @@ boxplot_data_frame <- function(
   yvar,
   xvar,
   facets = NULL,
-  log = FALSE,
-  horizontal = NULL,
-  crit = 1.3,
-  ref = NULL,
-  nobs = FALSE,
-  na.rm = TRUE,
+  log = getOption('metaplot_log',FALSE),
+  horizontal = getOption('metaplot_horizontal',NULL),
+  crit = getOption('metaplot_crit',1.3),
+  panel = getOption('metaplot_boxplot_panel',boxplot_panel),
+  ref = getOption('metaplot_ref',metaplot_ref),
+  ref.col = getOption('metaplot_ref.col','grey'),
+  ref.lty = getOption('metaplot_ref.lty','solid'),
+  ref.alpha = getOption('metaplot_ref.alpha',1),
+  nobs = getOption('metaplot_nobs',FALSE),
+  na.rm = getOption('metaplot_na.rm',TRUE),
   xlab = NULL,
   ylab = NULL,
-  aspect = 1,
+  numlab = getOption('metaplot_lab',axislabel),
+  catlab = getOption('metaplot_lab',axislabel),
+  aspect = getOption('metaplot_aspect',1),
   main = getOption('metaplot_main',NULL),
   sub = getOption('metaplot_sub',NULL),
   ...
@@ -69,8 +81,22 @@ boxplot_data_frame <- function(
   if(is.null(horizontal)) horizontal <- is.numeric(x[[xvar]]) & !encoded(xguide)
   cat <- if(horizontal) yvar else xvar
   con <- if(horizontal) xvar else yvar
+
+  if(is.character(ref)) ref <- match.fun(ref)
+  if(is.function(ref)) ref <- ref(x = x, var = con, ...)
+  ref <- as.numeric(ref)
+  ref <- ref[is.defined(ref)]
+
   stopifnot(all(c(cat,con) %in% names(y)))
-  if(na.rm) y %<>% filter(is.defined(UQ(cat)) & is.defined(UQ(con))) # preserves attributes
+  if(na.rm){
+    #y %<>% filter(is.defined(UQ(cat)) & is.defined(UQ(con))) # preserves attributes
+    foo <- y
+    y <- y[is.defined(y[[cat]]) & is.defined(y[[con]]),]
+    for(col in names(foo))attributes(y[[col]]) <- attributes(foo[[col]])
+    at <- attributes(foo)
+    at$row.names <- NULL
+    for(a in names(at)) attr(y,a) <- attr(foo,a)
+  }
   ff <- character(0)
   if(!is.null(facets))ff <- paste(facets, collapse = ' + ')
   if(!is.null(facets))ff <- paste0('|',ff)
@@ -98,10 +124,14 @@ boxplot_data_frame <- function(
     if(length(ref)) ref <- log10(ref)
     if(!length(ref)) ref <- NULL
   }
-  catlab <- axislabel(y,var = cat)
-  conlab <- axislabel(y,var = con, log = log)
-  if(is.null(ylab)) ylab <- if(horizontal) catlab else conlab
-  if(is.null(xlab)) xlab <- if(horizontal) conlab else catlab
+
+  if(is.character(catlab)) catlab <- tryCatch(match.fun(catlab), error = function(e)catlab)
+  if(is.function(catlab)) catlab <- catlab(x = y, var = cat, ...)
+  if(is.character(numlab)) numlab <- tryCatch(match.fun(numlab), error = function(e)numlab)
+  if(is.function(numlab)) numlab <- numlab(x = y, var = con, log = log, ...)
+
+  if(is.null(ylab)) ylab <- if(horizontal) catlab else numlab
+  if(is.null(xlab)) xlab <- if(horizontal) numlab else catlab
 
   guide <- attr(y[[cat]], 'guide')
   if(encoded(guide)){
@@ -123,16 +153,6 @@ boxplot_data_frame <- function(
     y = list(log = log,equispaced.log = FALSE)
   )
 
-  mypanel <- function(...){
-      panel.bwplot(...)
-    if(length(ref)){
-      if(horizontal) {
-        panel.abline(v = ref)
-      }else{
-        panel.abline(h = ref)
-      }
-    }
-  }
   if(!is.null(main))if(is.function(main)) main <- main(x = x, yvar = yvar, xvar = xvar, facets = facets, log = log, ...)
   if(!is.null(sub))if(is.function(sub)) sub <- sub(x = x, yvar = yvar, xvar = xvar, facets = facets, log = log, ...)
 
@@ -145,13 +165,43 @@ boxplot_data_frame <- function(
     scales = scales,
     ylab = ylab,
     xlab = xlab,
-    panel = mypanel,
     main = main,
     sub = sub,
+    panel = panel,
+    ref = ref,
+    ref.col = ref.col,
+    ref.lty = ref.lty,
+    ref.alpha = ref.alpha,
     ...
   )
 }
 
+#' Panel Function for Metaplot Boxplot
+#'
+#' Panel function for metaplot boxplot. Calls, \code{\link[lattice]{panel.bwplot}} and plots reference lines if ref has length.
+#' @export
+#' @family panel functions
+#' @family mixedvariate plots
+#' @keywords internal
+#' @param ref numeric
+#' @param horizontal passed to \code{\link[lattice]{panel.bwplot}}
+#' @param pch passed to \code{\link[lattice]{panel.bwplot}}
+#' @param notch passed to \code{\link[lattice]{panel.bwplot}}
+#' @param ref.col passed to \code{\link[lattice]{panel.abline}} as col
+#' @param ref.lty passed to \code{\link[lattice]{panel.abline}} as lty
+#' @param ref.alpha passed to \code{\link[lattice]{panel.abline}} as alpha
+#'
+boxplot_panel <- function(ref = NULL, horizontal,pch = '|',notch=FALSE, ref.col, ref.lty, ref.alpha, ...){
+  panel.bwplot(horizontal = horizontal, pch = pch, notch = notch, ...)
+  if(length(ref)){
+    if(horizontal) {
+      panel.abline(v = ref, col = ref.col, lty = ref.lty, alpha = ref.alpha)
+    }else{
+      panel.abline(h = ref, col = ref.col, lty = ref.lty, alpha = ref.alpha)
+    }
+  }
+}
+
 #' Boxplot Method for Data Frame
 #'
 #' Boxplot for data.frame.  Parses arguments and generates the call: fun(x, yvar, xvar, facets, ...).
@@ -162,7 +212,7 @@ boxplot_data_frame <- function(
 #' @importFrom rlang f_rhs
 #' @family mixedvariate plots
 #' @family boxplot
-#' @family metaplot
+#' @family methods
 #' @examples
 #' library(dplyr)
 #' library(magrittr)
