@@ -7,18 +7,20 @@
 #' @family generic functions
 #' @family univariate plots
 #' @family densplot
-#' @family metaplot
 densplot <- function(x,...)UseMethod('densplot')
 
 #' Density Function for Data Frame
 #'
-#' Plot density for object of class 'data.frame' using standard evaluation.
+#' Plot density for object of class 'data.frame' using \code{dens_panel} by default.
 #' @param x data.frame
 #' @param xvar variable to plot
 #' @param groups optional grouping variable
 #' @param facets optional conditioning variables
-#' @param xlab x axis label
-#' @param ref optional numeric
+#' @param xlab x axis label; can be function(x = x, var = xvar, log = log, ...)
+#' @param ref reference line; can be function(x = x, var = xvar, ...)
+#' @param ref.col color for reference line(s)
+#' @param ref.lty type for reference line(s)
+#' @param ref.alpha transparency for reference line(s)
 #' @param log whether to use log scale
 #' @param aspect passed to \code{\link[lattice]{densityplot}}
 #' @param scales  passed to \code{\link[lattice]{densityplot}}
@@ -30,6 +32,7 @@ densplot <- function(x,...)UseMethod('densplot')
 #' @param ... passed to \code{\link[lattice]{densityplot}}
 #' @family univariate plots
 #' @family densplot
+#' @family metaplot
 #' @import lattice
 #' @export
 #' @examples
@@ -41,14 +44,17 @@ densplot_data_frame<- function(
   xvar,
   groups = NULL,
   facets = NULL,
-  xlab = NULL,
-  ref = NULL,
-  log = FALSE,
-  aspect = 1,
-  scales = NULL,
-  panel = NULL,
-  auto.key = NULL,
-  keycols = NULL,
+  xlab = getOption('metaplot_lab',axislabel),
+  ref = getOption('metaplot_ref',metaplot_ref),
+  ref.col = getOption('metaplot_ref.col','grey'),
+  ref.lty = getOption('metaplot_ref.lty','solid'),
+  ref.alpha = getOption('metaplot_ref.alpha',1),
+  log = getOption('metaplot_log',FALSE),
+  aspect = getOption('metaplot_aspect',1),
+  scales = getOption('metaplot_dens.scales',NULL),
+  panel = getOption('metaplot_dens.panel',dens_panel),
+  auto.key = getOption('metaplot_auto.key',NULL),
+  keycols = getOption('metaplot_keycols',NULL),
   main = getOption('metaplot_main',NULL),
   sub = getOption('metaplot_sub',NULL),
   ...
@@ -61,14 +67,18 @@ densplot_data_frame<- function(
     log <- FALSE
   }
   if(is.null(scales)) scales <- list(tck = c(1,0),x = list(log = log,equispaced.log = FALSE))
-  if(is.null(panel)) panel <- function(ref = NULL, ...){
-    panel.densityplot(...)
-    if(length(ref))panel.abline(v = ref)
+
+  if(is.character(ref)) ref <- match.fun(ref)
+  if(is.function(ref)) ref <- ref(x = x, var = xvar, log = log, ...)
+  ref <- as.numeric(ref)
+  ref <- ref[is.defined(ref)]
+  if(log){
+    ref <- ref[ref > 0]
+    ref <- log(ref)
   }
-  default_xlab <- xvar
-  xvarlab <- attr(x[[xvar]],'label')
-  if(!is.null(xvarlab)) default_xlab <- xvarlab
-  if(is.null(xlab)) xlab <- default_xlab
+
+  if(is.character(xlab)) xlab <- tryCatch(match.fun(xlab), error = function(e)xlab)
+  if(is.function(xlab)) xlab <- xlab(x = x, var = xvar, log = log, ...)
 
   if(is.null(keycols))if(!is.null(groups))keycols <- min(3, length(unique(x[[groups]])))
   if(is.null(auto.key))if(!is.null(groups))if(length(unique(x[[groups]])) > 1) auto.key = list(columns = keycols,lines=TRUE)
@@ -92,6 +102,9 @@ densplot_data_frame<- function(
     groups = groups,
     xlab = xlab,
     ref = ref,
+    ref.col = ref.col,
+    ref.lty = ref.lty,
+    ref.alpha = ref.alpha,
     log = log,
     aspect = aspect,
     scales = scales,
@@ -102,6 +115,22 @@ densplot_data_frame<- function(
     ...
   )
 }
+
+#' Panel Function for Metaplot Density Plot
+#'
+#' Default panel function for dens_data_frame.  Calls panel.densityplot, and plots reference lines if ref has length.
+#' @export
+#' @family panel functions
+#' @family univariate plots
+#' @keywords internal
+#' @param ref numeric
+#' @param ref.col passed to \code{\link[lattice]{panel.abline}} as col
+#' @param ref.lty passed to \code{\link[lattice]{panel.abline}} as lty
+#' @param ref.alpha passed to \code{\link[lattice]{panel.abline}} as alpha
+dens_panel <- function(ref = NULL, ref.col, ref.lty, ref.alpha, ...){
+  panel.densityplot(...)
+  if(length(ref))panel.abline(v = ref, col=ref.col, lty = ref.lty, alpha = ref.alpha)
+}
 #' Densplot Method for Data Frame
 #'
 #' Plot density for object of class 'data.frame'. Parses arguments and generates the call: fun(x, xvar, groups, facets,...).
@@ -113,6 +142,7 @@ densplot_data_frame<- function(
 #' @importFrom rlang f_rhs quos
 #' @family univariate plots
 #' @family densplot
+#' @family methods
 #' @examples
 #' densplot(Theoph, conc, grid = TRUE )
 #' densplot(Theoph, conc, Subject )
